# Generated by Django 4.2.8 on 2024-02-16 10:25

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_bgp', '0031_netbox_bgp'),
    ]

    operations = [
        migrations.AddField(
            model_name='bgpsession',
            name='prefix_list_in',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='session_prefix_in', to='netbox_bgp.prefixlist'),
        ),
        migrations.AddField(
            model_name='bgpsession',
            name='prefix_list_out',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='session_prefix_out', to='netbox_bgp.prefixlist'),
        ),
    ]
