# Generated by Django 4.1.9 on 2023-09-19 05:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_bgp', '0028_netbox_bgp'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bgpsession',
            name='name',
            field=models.CharField(blank=True, max_length=256, null=True),
        ),
    ]
