# Generated by Django 5.2.1 on 2025-05-19 23:35

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0207_remove_redundant_indexes'),
        ('ipam', '0081_remove_service_device_virtual_machine_add_parent_gfk_index'),
        ('netbox_bgp', '0032_netbox_bgp'),
        ('virtualization', '0048_populate_mac_addresses'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='bgpsession',
            unique_together={('device', 'local_address', 'local_as', 'remote_address', 'remote_as')},
        ),
        migrations.AddField(
            model_name='bgpsession',
            name='virtualmachine',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='virtualization.virtualmachine'),
        ),
        migrations.AlterUniqueTogether(
            name='bgpsession',
            unique_together={('device', 'local_address', 'local_as', 'remote_address', 'remote_as'), ('virtualmachine', 'local_address', 'local_as', 'remote_address', 'remote_as')},
        ),
    ]
