# Generated by Django 4.2.7 on 2023-12-14 09:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_bgp', '0029_netbox_bgp'),
    ]

    operations = [
        migrations.AddField(
            model_name='bgppeergroup',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='bgpsession',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='community',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='prefixlist',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='prefixlistrule',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='prefixlistrule',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='routingpolicy',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='routingpolicyrule',
            name='comments',
            field=models.TextField(blank=True),
        ),
    ]
